@@ -63,7 +63,7 @@ __all__ = ["backpropagate_2d", "fourier_map_2d", "sum_2d"]
 _verbose = 1
 
 
-def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
+def backpropagate_2d(uSin, angles, res, nm, lD=0, coords=None,
                      weight_angles=True,
                      onlyreal=False, padding=True, padval=0,
                      jmc=None, jmm=None, verbose=_verbose):
@@ -75,28 +75,40 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
     by a dielectric object with refractive index
     :math:`n(x,z)`.
 
-    This method implements the backpropagation formula:
+    This method implements the 2D backpropagation algorithm [1]_
 
     .. math::
         f(\mathbf{r}) = 
-            - \\frac{i k_\mathrm{m}}{a_0 (2 \pi)^{3/2}}
-            \\int \!\! dk_\mathrm{Dx} \\int_0^{2 \pi} \!\!  d\phi_0 \,
+            -\\frac{i k_\mathrm{m}}{2\pi}
+            \\sum_{j=1}^{N} \! \Delta \phi_0 D_{-\phi_j} \!\!
+            \\left \{
+            \\text{FFT}^{-1}_{\mathrm{1D}}
+            \\left \{
             \\left| k_\mathrm{Dx} \\right|  
-            \widehat{U}_{\mathrm{B},\phi_0}(k_\mathrm{Dx})
-            \exp( -i k_\mathrm{m} M l_\mathrm{D} )  
-            \exp \! \\left[ i (k_\mathrm{Dx} \, \mathbf{t_\\perp} 
-            + k_\mathrm{m}(M - 1) \, \mathbf{s_0})\mathbf{r} \\right] 
+            \\frac{\\text{FFT}_{\mathrm{1D}} \\left \{
+            u_{\mathrm{B},\phi_j}(x_\mathrm{D}) \\right \}
+            }{u_0(l_\mathrm{D})}
+            \exp \! \\left[i k_\mathrm{m}(M - 1) \cdot 
+            (z_{\phi_j}-l_\mathrm{D}) \\right]
+            \\right \} 
+            \\right \}
 
+    with the forward :math:`\\text{FFT}_{\mathrm{1D}}` and inverse 
+    :math:`\\text{FFT}^{-1}_{\mathrm{1D}}` 1D fast Fourier transform, the
+    rotational operator :math:`D_{-\phi_j}`, the angular distance between the
+    projections :math:`\Delta \phi_0`, the ramp filter in Fourier space
+    :math:`|k_\mathrm{Dx}|`, and the propagation distance 
+    :math:`(z_{\phi_j}-l_\mathrm{D})`.
 
     Parameters
     ----------
     uSin : (A,N) ndarray
         Two-dimensional sinogram of line recordings
-        :math:`u_{\mathrm{B}, \phi_0}(x_\mathrm{D})`
-        normalized by the amplitude of the unscattered wave :math:`a_0`
+        :math:`u_{\mathrm{B}, \phi_j}(x_\mathrm{D})`
+        divided by the incident plane wave :math:`u_0(l_\mathrm{D})`
         measured at the detector.
     angles : (A,) ndarray
-        Angular positions :math:`\phi_0` of ``uSin`` in radians.
+        Angular positions :math:`\phi_j` of ``uSin`` in radians.
     res : float
         Vacuum wavelength of the light :math:`\lambda` in pixels.
     nm : float
@@ -104,20 +116,23 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
     lD : float
         Distance from center of rotation to detector plane 
         :math:`l_\mathrm{D}` in pixels.
-    coords : None [(2,M) ndarray], optional
+    coords : None [(2,M) ndarray]
         Computes only the output image at these coordinates. This
         keyword is reserved for future versions and is not
         implemented yet.
-    weight_angles : bool, optional
-        If `True` weight each backpropagated projection with a factor
+    weight_angles : bool
+        If ``True``, weights each backpropagated projection with a factor
         proportional to the angular distance between the neighboring
-        projections.
+        projections. 
+        
+        .. math::
+            \Delta \phi_0 \\longmapsto \Delta \phi_j = \\frac{\phi_{j+1} - \phi_{j-1}}{2}
         
         .. versionadded:: 0.1.1
-    onlyreal : bool, optional
-        If `True`, only the real part of the reconstructed image
+    onlyreal : bool
+        If ``True``, only the real part of the reconstructed image
         will be returned. This saves computation time.
-    padding : bool, optional
+    padding : bool
         Pad the input data to the second next power of 2 before
         Fourier transforming. This reduces artifacts and speeds up
         the process for input image sizes that are not powers of 2.
@@ -126,12 +141,12 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
         approximation, where an approximate zero in the phase might
         translate to 2πi due to the unwrapping algorithm. In that
         case, this value should be a multiple of 2πi. 
-        If `padval` is `None`, then the edge values are used for
-        padding (see documentation of `numpy.pad`).
-    jmc, jmm : instance of :func:`multiprocessing.Value` or `None`
+        If ``padval`` is ``None``, then the edge values are used for
+        padding (see documentation of :func:`numpy.pad`).
+    jmc, jmm : instance of :func:`multiprocessing.Value` or ``None``
         The progress of this function can be monitored with the 
-        :mod:`jobmanager` package. The current step `jmc.value` is
-        incremented `jmm.value` times. `jmm.value` is set at the 
+        :mod:`jobmanager` package. The current step ``jmc.value`` is
+        incremented ``jmm.value`` times. ``jmm.value`` is set at the 
         beginning.
     verbose : int
         Increment to increase verbosity.
@@ -153,6 +168,15 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
 
     radontea.backproject : backprojection based on the Fourier slice
         theorem.
+    
+    Notes
+    -----
+    Do not use the parameter `lD` in combination with the Rytov
+    approximation - the propagation is not correctly described.
+    Instead, numerically refocus the sinogram prior to converting
+    it to Rytov data (using e.g. :func:`odtbrain.sinogram_as_rytov`)
+    with a numerical focusing algorithm (available in the Python
+    package :py:mod:`nrefocus`).
     """
     ##
     ##
@@ -165,10 +189,9 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
     if jmm is not None:
         jmm.value = A + 2
     # Check input data
-    if len(uSin.shape) != 2:
-        raise ValueError("Input data `uB` must have shape (A,N).")
-    if len(uSin) != A:
-        raise ValueError("`len(angles)` must be  equal to `len(uSin)`.")
+    assert len(uSin.shape) == 2, "Input data `uB` must have shape (A,N)!"
+    assert len(uSin) == A, "`len(angles)` must be  equal to `len(uSin)`!"
+    
     if coords is not None:
         raise NotImplementedError("Output coordinates cannot yet" +
                                   " be set for the 2D backrpopagation algorithm.")
@@ -285,12 +308,14 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
     # Filter M so there are no nans from the root
     M = 1. / km * np.sqrt((km**2 - kx**2) * filter_klp)
 
-    # We already divided uB by a0 at the very beginning
-    #prefactor  = -1j * km / ( 2 * np.pi * a0 )
     prefactor = -1j * km / (2 * np.pi)
     prefactor *= dphi0
     prefactor *= np.abs(kx) * filter_klp
-    prefactor *= np.exp(-1j * km * M * lD)
+    # new in version 0.1.4:
+    # We multiply by the factor (M-1) instead of just (M)
+    # to take into account that we have a scattered
+    # wave that is normalized by u0.
+    prefactor *= np.exp(-1j * km * (M-1) * lD)
     # Perform filtering of the sinogram
     projection = np.fft.fft(sino, axis=-1) * prefactor
 
@@ -358,7 +383,7 @@ def backpropagate_2d(uSin, angles, res, nm, lD, coords=None,
     return outarr
 
 
-def fourier_map_2d(uSin, angles, res, nm, lD, semi_coverage=False,
+def fourier_map_2d(uSin, angles, res, nm, lD=0, semi_coverage=False,
                    coords=None, jmc=None, jmm=None, verbose=_verbose):
     u""" 2D Fourier mapping with the Fourier diffraction theorem
 
@@ -375,11 +400,11 @@ def fourier_map_2d(uSin, angles, res, nm, lD, semi_coverage=False,
     ----------
     uSin : (A,N) ndarray
         Two-dimensional sinogram of line recordings
-        :math:`u_{\mathrm{B}, \phi_0}(x_\mathrm{D})`
-        normalized by the amplitude of the unscattered wave :math:`a_0`
+        :math:`u_{\mathrm{B}, \phi_j}(x_\mathrm{D})`
+        divided by the incident plane wave :math:`u_0(l_\mathrm{D})`
         measured at the detector.
     angles : (A,) ndarray
-        Angular positions :math:`\phi_0` of ``uSin`` in radians.
+        Angular positions :math:`\phi_j` of ``uSin`` in radians.
     res : float
         Vacuum wavelength of the light :math:`\lambda` in pixels.
     nm : float
@@ -387,21 +412,21 @@ def fourier_map_2d(uSin, angles, res, nm, lD, semi_coverage=False,
     lD : float
         Distance from center of rotation to detector plane 
         :math:`l_\mathrm{D}` in pixels.
-    semi_coverage : bool, optional
-        If set to `True`, it is assumed that the sinogram does not 
+    semi_coverage : bool
+        If set to ``True``, it is assumed that the sinogram does not 
         necessarily cover the full angular range from 0 to 2π, but an
         equidistant coverage over 2π can be achieved by inferring point
         (anti)symmetry of the (imaginary) real parts of the Fourier 
         transform of f. Valid for any set of angles {X} that result in
         a 2π coverage with the union set {X}U{X+π}.
-    coords : None [(2,M) ndarray], optional
+    coords : None [(2,M) ndarray]
         Computes only the output image at these coordinates. This
         keyword is reserved for future versions and is not
         implemented yet.
-    jmc, jmm : instance of :func:`multiprocessing.Value` or `None`
+    jmc, jmm : instance of :func:`multiprocessing.Value` or ``None``
         The progress of this function can be monitored with the 
-        :mod:`jobmanager` package. The current step `jmc.value` is
-        incremented `jmm.value` times. `jmm.value` is set at the 
+        :mod:`jobmanager` package. The current step ``jmc.value`` is
+        incremented ``jmm.value`` times. ``jmm.value`` is set at the 
         beginning.
     verbose : int
         Increment to increase verbosity.
@@ -422,6 +447,14 @@ def fourier_map_2d(uSin, angles, res, nm, lD, semi_coverage=False,
     odt_to_ri : conversion of the object function :math:`f(\mathbf{r})` 
         to refractive index :math:`n(\mathbf{r})`.
 
+    Notes
+    -----
+    Do not use the parameter `lD` in combination with the Rytov
+    approximation - the propagation is not correctly described.
+    Instead, numerically refocus the sinogram prior to converting
+    it to Rytov data (using e.g. :func:`odtbrain.sinogram_as_rytov`)
+    with a numerical focusing algorithm (available in the Python
+    package :py:mod:`nrefocus`).
     """
     ##
     ##
@@ -437,10 +470,9 @@ def fourier_map_2d(uSin, angles, res, nm, lD, semi_coverage=False,
     if jmm is not None:
         jmm.value = 4
     # Check input data
-    if len(uSin.shape) != 2:
-        raise ValueError("Input data `uSin` must have shape (A,N).")
-    if len(uSin) != A:
-        raise ValueError("`len(angles)` must be  equal to `len(uSin)`.")
+    assert len(uSin.shape) == 2, "Input data `uSin` must have shape (A,N)!"
+    assert len(uSin) == A, "`len(angles)` must be  equal to `len(uSin)`!"
+    
     if coords is not None:
         raise NotImplementedError("Output coordinates cannot yet" +
                                   " be set for the 2D backrpopagation algorithm.")
@@ -542,8 +574,11 @@ def fourier_map_2d(uSin, angles, res, nm, lD, semi_coverage=False,
     filter_klp = (kx**2 < km**2)
     M = 1. / km * np.sqrt(km**2 - kx**2)
     #Fsin =  -1j * km * np.sqrt(2/np.pi) / a0 * M * np.exp(-1j*km*M*lD)
-    # We divided by a0 before.
-    Fsin = -1j * km * np.sqrt(2 / np.pi) * M * np.exp(-1j * km * M * lD)
+    # new in version 0.1.4:
+    # We multiply by the factor (M-1) instead of just (M)
+    # to take into account that we have a scattered
+    # wave that is normalized by u0.
+    Fsin = -1j * km * np.sqrt(2 / np.pi) * M * np.exp(-1j * km * (M-1) * lD)
 
     # UB has same shape (len(angles), len(kx))
     Fsin = Fsin * UB * filter_klp
@@ -606,7 +641,7 @@ def fourier_map_2d(uSin, angles, res, nm, lD, semi_coverage=False,
     return f[::-1]
 
 
-def sum_2d(uSin, angles, res, nm, lD, coords=None,
+def sum_2d(uSin, angles, res, nm, lD=0, coords=None,
            jmc=None, jmm=None, verbose=_verbose):
     u""" 2D sum-reconstruction with the Fourier diffraction theorem
 
@@ -623,11 +658,11 @@ def sum_2d(uSin, angles, res, nm, lD, coords=None,
     ----------
     uSin : (A,N) ndarray
         Two-dimensional sinogram of line recordings
-        :math:`u_{\mathrm{B}, \phi_0}(x_\mathrm{D})`
-        normalized by the amplitude of the unscattered wave :math:`a_0`
+        :math:`u_{\mathrm{B}, \phi_j}(x_\mathrm{D})`
+        divided by the incident plane wave :math:`u_0(l_\mathrm{D})`
         measured at the detector.
     angles : (A,) ndarray
-        Angular positions :math:`\phi_0` of ``uSin`` in radians.
+        Angular positions :math:`\phi_j` of ``uSin`` in radians.
     res : float
         Vacuum wavelength of the light :math:`\lambda` in pixels.
     nm : float
@@ -635,14 +670,14 @@ def sum_2d(uSin, angles, res, nm, lD, coords=None,
     lD : float
         Distance from center of rotation to detector plane 
         :math:`l_\mathrm{D}` in pixels.
-    coords : None or (2,M) ndarray], optional
+    coords : None or (2,M) ndarray]
         Computes only the output image at these coordinates. This
         keyword is reserved for future versions and is not
         implemented yet.
-    jmc, jmm : instance of :func:`multiprocessing.Value` or `None`
+    jmc, jmm : instance of :func:`multiprocessing.Value` or ``None``
         The progress of this function can be monitored with the 
-        :mod:`jobmanager` package. The current step `jmc.value` is
-        incremented `jmm.value` times. `jmm.value` is set at the 
+        :mod:`jobmanager` package. The current step ``jmc.value`` is
+        incremented ``jmm.value`` times. ``jmm.value`` is set at the 
         beginning.
     verbose : int
         Increment to increase verbosity.
@@ -671,6 +706,12 @@ def sum_2d(uSin, angles, res, nm, lD, coords=None,
     is included in the package, because of its educational value,
     exemplifying the backpropagation algorithm.
 
+    Do not use the parameter `lD` in combination with the Rytov
+    approximation - the propagation is not correctly described.
+    Instead, numerically refocus the sinogram prior to converting
+    it to Rytov data (using e.g. :func:`odtbrain.sinogram_as_rytov`)
+    with a numerical focusing algorithm (available in the Python
+    package :py:mod:`nrefocus`).
     """
     if coords is None:
         lx = uSin.shape[1]
@@ -770,7 +811,11 @@ def sum_2d(uSin, angles, res, nm, lD, coords=None,
     # Also filter the prefactor, so nothing outside the required
     # low-pass contributes to the sum.
     prefactor *= np.abs(kx) * filter_klp
-    prefactor *= np.exp(-1j * km * M * lD)
+    # new in version 0.1.4:
+    # We multiply by the factor (M-1) instead of just (M)
+    # to take into account that we have a scattered
+    # wave that is normalized by u0.
+    prefactor *= np.exp(-1j * km * (M-1) * lD)
 
     # Initiate function f
     f = np.zeros(len(coords[0]), dtype=np.complex128)
