@@ -65,6 +65,7 @@ from __future__ import division, print_function
 import ctypes
 import gc
 import multiprocessing as mp
+import numexpr as ne
 import numpy as np
 import platform
 import pyfftw
@@ -130,7 +131,6 @@ def _mprotate(ang, lny, pool, order):
 
 def _rotate(d):
     (ymin, ymax, ang, order) = d
-    # print(_ang.value)
     return scipy.ndimage.interpolation.rotate(
         odtbrain._shared_array[:, ymin:ymax, :],  # input
         angle=-ang,  # angle
@@ -141,20 +141,14 @@ def _rotate(d):
         mode="constant",  # mode
         cval=0)
 
-def _filter2_func(args):
-    """
-    computes exp(1j * z * km*(M-1))
-    
-    """
-    zvp, Mpm1 = args
-    return np.exp(1j * zvp * Mpm1)
 
-
-def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
+def backpropagate_3d(uSin, angles, res, nm, lD=0, coords=None,
                      weight_angles=True, onlyreal=False, 
                      padding=(True, True), padfac=1.75, padval=None,
                      intp_order=2, dtype=_np_float64,
-                     num_cores=_ncores, 
+                     num_cores=_ncores,
+                     save_memory=False,
+                     copy=True,
                      jmc=None, jmm=None,
                      verbose=_verbose):
     u""" 3D backpropagation with the Fourier diffraction theorem
@@ -165,29 +159,41 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
     by a dielectric object with refractive index
     :math:`n(x,y,z)`.
 
-    This method implements the 3D backpropagation formula:
+    This method implements the 3D backpropagation algorithm [1]_
+
 
     .. math::
-        f(\mathbf{r}) &= \\frac{-ik_\mathrm{m}}{(2\pi)^{2}a_\mathrm0} 
-            \int_0^{2\pi} \!\! d\phi_0
-            \int_{-k_\mathrm{m}}^{k_\mathrm{m}} \!\! dk_\mathrm{Dx}
-            \int_{-k_\mathrm{m}}^{k_\mathrm{m}} \!\! dk_\mathrm{Dy} \,
-            \\left| k_\mathrm{Dx} \\right| 
-            \widehat{U}_{\mathrm{B},\phi_0}(k_\mathrm{Dx},k_\mathrm{Dy})
-            \exp(-ik_\mathrm{m}M l_\mathrm{D})
-            \exp[i(k_\mathrm{Dx} \, \mathbf{t_\\perp} + k_\mathrm{m}(M - 1) \, \mathbf{s_0})\mathbf{r}]
+        f(\mathbf{r}) = 
+            -\\frac{i k_\mathrm{m}}{2\pi}
+            \\sum_{j=1}^{N} \! \Delta \phi_0 D_{-\phi_j} \!\!
+            \\left \{
+            \\text{FFT}^{-1}_{\mathrm{2D}}
+            \\left \{
+            \\left| k_\mathrm{Dx} \\right|  
+            \\frac{\\text{FFT}_{\mathrm{2D}} \\left \{
+            u_{\mathrm{B},\phi_j}(x_\mathrm{D}, y_\mathrm{D}) \\right \}}
+            {u_0(l_\mathrm{D})}
+            \exp \! \\left[i k_\mathrm{m}(M - 1) \cdot
+            (z_{\phi_j}-l_\mathrm{D}) \\right]
+            \\right \} 
+            \\right \}
 
+    with the forward :math:`\\text{FFT}_{\mathrm{2D}}` and inverse 
+    :math:`\\text{FFT}^{-1}_{\mathrm{2D}}` 2D fast Fourier transform, the
+    rotational operator :math:`D_{-\phi_j}`, the angular distance between the
+    projections :math:`\Delta \phi_0`, the ramp filter in Fourier space
+    :math:`|k_\mathrm{Dx}|`, and the propagation distance 
+    :math:`(z_{\phi_j}-l_\mathrm{D})`.
 
     Parameters
     ----------
     uSin : (A, Ny, Nx) ndarray
         Three-dimensional sinogram of plane recordings
-        :math:`u_{\mathrm{B}, \phi_0}(x_\mathrm{D}, y_\mathrm{D},
-        z_\mathrm{D})`
-        normalized by the amplitude of the unscattered wave :math:`a_0`
+        :math:`u_{\mathrm{B}, \phi_j}(x_\mathrm{D}, y_\mathrm{D})`
+        divided by the incident plane wave :math:`u_0(l_\mathrm{D})`
         measured at the detector.
     angles : (A,) ndarray
-        Angular positions :math:`\phi_0` of ``uSin`` in radians.
+        Angular positions :math:`\phi_j` of ``uSin`` in radians.
     res : float
         Vacuum wavelength of the light :math:`\lambda` in pixels.
     nm : float
@@ -199,23 +205,26 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
         Only compute the output image at these coordinates. This
         keyword is reserved for future versions and is not
         implemented yet.
-    weight_angles : bool, optional
-        If `True` weight each backpropagated projection with a factor
+    weight_angles : bool
+        If ``True``, weights each backpropagated projection with a factor
         proportional to the angular distance between the neighboring
-        projections.
+        projections. 
+        
+        .. math::
+            \Delta \phi_0 \\longmapsto \Delta \phi_j = \\frac{\phi_{j+1} - \phi_{j-1}}{2}
         
         .. versionadded:: 0.1.1
     onlyreal : bool
-        If `True`, only the real part of the reconstructed image
+        If ``True``, only the real part of the reconstructed image
         will be returned. This saves computation time.
     padding : tuple of bool
         Pad the input data to the second next power of 2 before
         Fourier transforming. This reduces artifacts and speeds up
         the process for input image sizes that are not powers of 2.
-        The default is padding in x and y: `padding=(True, True)`.
+        The default is padding in x and y: ``padding=(True, True)``.
         For padding only in x-direction (e.g. for cylindrical
-        symmetries), set `padding` to `(True, False)`. To turn off
-        padding, set it to `(False, False)`.
+        symmetries), set ``padding`` to ``(True, False)``. To turn off
+        padding, set it to ``(False, False)``.
     padfac : float
         Increase padding size of the input data. A value greater
         than one will trigger padding to the second-next power of
@@ -229,21 +238,32 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
         approximation, where an approximat zero in the phase might
         translate to 2πi due to the unwrapping algorithm. In that
         case, this value should be a multiple of 2πi. 
-        If `padval` is `None`, then the edge values are used for
-        padding (see documentation of `numpy.pad`).
+        If ``padval`` is ``None``, then the edge values are used for
+        padding (see documentation of :func:`numpy.pad`).
     order : int between 0 and 5
         Order of the interpolation for rotation.
         See :func:`scipy.ndimage.interpolation.rotate` for details.
-    dtype : dtype object or argument for np.dtype
+    dtype : dtype object or argument for :func:`numpy.dtype`
         The data type that is used for calculations (float or double).
-        Defaults to np.float.
+        Defaults to ``numpy.float``.
     num_cores : int
         The number of cores to use for parallel operations. This value
         defaults to the number of cores on the system.
-    jmc, jmm : instance of :func:`multiprocessing.Value` or `None`
+    save_memory : bool
+        Saves memory at the cost of longer computation time.
+        
+        .. versionadded:: 0.1.5
+        
+    copy : bool
+        Copy input sinogram ``uSin`` for data processing. If ``copy``
+        is set to ``False``, then ``uSin`` will be overridden.
+        
+        .. versionadded:: 0.1.5
+        
+    jmc, jmm : instance of :func:`multiprocessing.Value` or ``None``
         The progress of this function can be monitored with the 
-        :mod:`jobmanager` package. The current step `jmc.value` is
-        incremented `jmm.value` times. `jmm.value` is set at the 
+        :mod:`jobmanager` package. The current step ``jmc.value`` is
+        incremented ``jmm.value`` times. ``jmm.value`` is set at the 
         beginning.
     verbose : int
         Increment to increase verbosity.
@@ -263,17 +283,28 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
     odt_to_ri : conversion of the object function :math:`f(\mathbf{r})` 
         to refractive index :math:`n(\mathbf{r})`.
 
+    Notes
+    -----
+    Do not use the parameter `lD` in combination with the Rytov
+    approximation - the propagation is not correctly described.
+    Instead, numerically refocus the sinogram prior to converting
+    it to Rytov data (using e.g. :func:`odtbrain.sinogram_as_rytov`)
+    with a numerical focusing algorithm (available in the Python
+    package :py:mod:`nrefocus`).
     """
+    ne.set_num_threads(num_cores)
+
+    if copy:
+        uSin = uSin.copy()
+
     A = angles.shape[0]
     # jobmanager
     if jmm is not None:
         jmm.value = A + 2
     
-    
     # check for dtype
     dtype = np.dtype(dtype)
-    if not dtype.name in ["float32", "float64"]:
-        raise ValueError("dtype must be float32 or float64.")
+    assert dtype.name in ["float32", "float64"], "dtype must be float32 or float64!"
 
     assert num_cores <= _ncores, "`num_cores` must not exceed number " +\
                                  "of physical cores: {}".format(_ncores)
@@ -312,13 +343,11 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
     # This is not a big problem. We only need to multiply the imaginary
     # part of the scattered wave by -1.
 
+    sinogram = uSin
     # Perform weighting
     if weight_angles:
         weights = util.compute_angle_weights_1d(angles).reshape(-1,1,1)
-        sinogram = weights * uSin
-    else:
-        # save memory
-        sinogram = uSin
+        sinogram *= weights
 
     # lengths of the input data
     (la, lny, lnx) = sinogram.shape
@@ -367,6 +396,7 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
 
     # save memory
     del sinogram
+
     if verbose > 0:
         print("......Image size (x,y): {}x{}, padded: {}x{}".format(
             lnx, lny, sino.shape[2], sino.shape[1]))
@@ -431,15 +461,18 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
 
     # Filter M so there are no nans from the root
     M = 1. / km * np.sqrt((km**2 - kx**2 - ky**2) * filter_klp)
-    # The input data is already divided by a0
-    #prefactor  = -1j * km / ( 2 * np.pi * a0 )
+
     prefactor = -1j * km / (2 * np.pi)
     prefactor *= dphi0
     # Also filter the prefactor, so nothing outside the required
     # low-pass contributes to the sum.
     prefactor *= np.abs(kx) * filter_klp
     #prefactor *= np.sqrt(((kx**2+ky**2)) * filter_klp )
-    prefactor *= np.exp(-1j * km * M * lD)
+    # new in version 0.1.4:
+    # We multiply by the factor (M-1) instead of just (M)
+    # to take into account that we have a scattered
+    # wave that is normalized by u0.
+    prefactor *= np.exp(-1j * km * (M-1) * lD)
     # Perform filtering of the sinogram,
     # save memory by in-place operations
     #projection = np.fft.fft2(sino, axes=(-1,-2)) * prefactor
@@ -468,7 +501,8 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
     temp_array, myfftw_plan
 
     projection = sino
-    projection[:] *= prefactor
+    # - normalize to (lNx * lNy) for FFTW
+    projection[:] *= prefactor / (lNx * lNy)
 
     # save memory
     del prefactor, filter_klp
@@ -497,22 +531,19 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
     #              z, y, x
     Mp = M.reshape(lNy, lNx)
 
-
-    # Compute the filter in Fourier space in parallel on-by one
-    # This saves an enormous amount of memory when compared to
-    # simply executing:
     # filter2 = np.exp(1j * zv * km * (Mp - 1))
-    
-    Mpm1 = km * (Mp - 1)
-    args=zip(zv.flatten(), [Mpm1]*zv.shape[0])
-    filter2_pool = mp.Pool(processes=num_cores)
-    filter2 = filter2_pool.map(_filter2_func, args)
-    filter2_pool.terminate()
-    filter2_pool.terminate()
-    del filter2_pool, args, Mpm1
-
-    # occupies some amount of ram
-    #filter2[0].size*len(filter2)*128/8/1024**3
+    f2_exp_fac = 1j* km * (Mp - 1)
+    if save_memory:
+        # compute filter2 later
+        pass
+    else:
+        # compute filter2 now
+        filter2 = ne.evaluate("exp(factor * zv)",
+                              local_dict={"factor": f2_exp_fac,
+                                          "zv":zv})
+        # occupies some amount of ram, but yields faster
+        # computation later
+        #filter2[0].size*len(filter2)*128/8/1024**3
 
     if jmc is not None:
         jmc.value += 1
@@ -562,7 +593,7 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
     # filtered projections in loop
     filtered_proj = np.zeros((ln, lny, lnx), dtype=dtype_complex)
 
-    for i in np.arange(A):
+    for aa in np.arange(A):
         # 14x Speedup with fftw3 compared to numpy fft and
         # memory reduction by a factor of 2!
         # ifft will be computed in-place
@@ -571,12 +602,22 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
         # projection.shape == (A, lNx, lNy)
         # filter2.shape == (ln, lNx, lNy)
         for p in range(len(zv)):
-            inarr[:] = filter2[p] * projection[i]
+            if save_memory:
+                # compute filter2 here;
+                # this is comparatively slower than the other case
+                ne.evaluate("exp(factor * zvp) * projectioni",
+                            local_dict={"zvp":zv[p],
+                                        "projectioni":projection[aa],
+                                        "factor":f2_exp_fac},
+                            out=inarr)
+            else:
+                # use universal functions
+                np.multiply(filter2[p], projection[aa], out=inarr)
             myifftw_plan.execute()
             filtered_proj[p, :, :] = inarr[
                                        padyl:padyl + lny,
                                        padxl:padxl + lnx
-                                      ] / (lNx * lNy)
+                                      ]
 
         # resize image to original size
         # The copy is necessary to prevent memory leakage.
@@ -584,15 +625,13 @@ def backpropagate_3d(uSin, angles, res, nm, lD, coords=None,
         #_shared_array[:] = sino_filtered.real[:ln, :lny, :lnx] / (lNx * lNy)
         # By performing the "/" operation here, we magically use less
         # memory and we gain speed...
-        _shared_array[:] = filtered_proj.real[:]
+        _shared_array[:] = filtered_proj.real
         #_shared_array[:] = sino_filtered.real[ :ln, padyl:padyl + lny, padxl:padxl + lnx] / (lNx * lNy)
 
-        phi0 = np.rad2deg(angles[i])
+        phi0 = np.rad2deg(angles[aa])
 
         if not onlyreal:
             filtered_proj_imag = filtered_proj.imag
-        else:
-            filtered_proj_imag = None
 
         _mprotate(phi0, lny, pool4loop, intp_order)
 
