@@ -5,6 +5,7 @@ from __future__ import division, print_function
 import ctypes
 import gc
 import multiprocessing as mp
+import numexpr as ne
 import numpy as np
 from numpy import cos, sin
 
@@ -12,7 +13,7 @@ import pyfftw
 import scipy.ndimage
 import warnings
 
-from ._Back_3D import _ncores, _np_float64, _verbose, _filter2_func
+from ._Back_3D import _ncores, _np_float64, _verbose
 from . import util
 import odtbrain
 
@@ -234,7 +235,6 @@ def rotation_matrix_from_point_planerot(point, plane_angle, ret_inv=False):
         return DR
 
 
-
 def sphere_points_from_angles_and_tilt(angles, tilted_axis):
     """
     For a given tilt of the rotational axis `tilted_axis`, compute
@@ -252,8 +252,11 @@ def sphere_points_from_angles_and_tilt(angles, tilted_axis):
     Notes
     -----
     The reference axis is always [0,1,0].
+    `theta` is the azimuthal angle measured down from the y-axis.
+    `phi` is the polar angle in the x-z plane measured from z towards x.
 
     """
+    assert len(angles.shape) == 1 
     ## Normalize tilted axis.
     tilted_axis = norm_vec(tilted_axis)
     [u, v, w] = tilted_axis
@@ -270,7 +273,7 @@ def sphere_points_from_angles_and_tilt(angles, tilted_axis):
     #   z-axis pointing right
     # - The rotation of `tilted_axis` can be described by two
     #   separate rotations. We will use these two angles:
-    #   (a) Rotation from y=1 within the y-z plane:
+    #   (a) Rotation from y=1 within the y-z plane: theta
     #       This is the rotation that is critical for data
     #       reconstruction. If this angle is zero, then we
     #       have a rotational axis in the imaging plane. If
@@ -278,7 +281,7 @@ def sphere_points_from_angles_and_tilt(angles, tilted_axis):
     #       of a rotating image and 3D reconstruction is
     #       impossible. This angle is counted from the y-axis
     #       onto the x-z plane.
-    #   (b) Rotation in the x-z plane:
+    #   (b) Rotation in the x-z plane: phi
     #       This angle is responsible for matching up the angles
     #       with the correct sinogram images. If this angle is zero,
     #       then the projection of the rotational axis onto the
@@ -290,11 +293,11 @@ def sphere_points_from_angles_and_tilt(angles, tilted_axis):
     #       that lies in the x-z plane. angles[1] is the next point
     #       towards the x-axis if phi==0.
     
-    # (a) This angle is the polar angle theta measured from the
+    # (a) This angle is the azimuthal angle theta measured from the
     #     y-axis.
     theta = np.arccos(v)
     
-    # (b) This is the angle measured in the x-z plane starting
+    # (b) This is the polar angle measured in the x-z plane starting
     #     at the x-axis and measured towards the positive z-axis.
     phi = np.arctan2(u, w)
     
@@ -370,16 +373,15 @@ def sphere_points_from_angles_and_tilt(angles, tilted_axis):
     return newang
 
 
-
-    
-
-def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
+def backpropagate_3d_tilted(uSin, angles, res, nm, lD=0,
                      tilted_axis=[0, 1, 0],
                      coords=None, weight_angles=True, onlyreal=False,
                      offset_alpha=0, offset_beta=0,
                      padding=(True, True), padfac=1.75, padval=None,
                      intp_order=2, dtype=_np_float64,
-                     num_cores=_ncores, 
+                     num_cores=_ncores,
+                     save_memory=False,
+                     copy=True,
                      jmc=None, jmm=None,
                      verbose=_verbose):
     u""" 3D backpropagation with the Fourier diffraction theorem
@@ -390,25 +392,47 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
     :math:`n(x,y,z)`.
     
     This method implements the 3D backpropagation algorithm with
-    a rotational axis that is tilted w.r.t. the imaging plane.
+    a rotational axis that is tilted by :math:`\\theta_\mathrm{tilt}`
+    w.r.t. the imaging plane [3]_
 
+    .. math::
+        f(\mathbf{r}) = 
+            -\\frac{i k_\mathrm{m}}{2\pi}
+            \\sum_{j=1}^{N} \! \Delta \phi_0 D_{-\phi_j}^\mathrm{tilt} \!\!
+            \\left \{
+            \\text{FFT}^{-1}_{\mathrm{2D}}
+            \\left \{
+            \\left| k_\mathrm{Dx} \cdot \cos \\theta_\mathrm{tilt}\\right|  
+            \\frac{\\text{FFT}_{\mathrm{2D}} \\left \{
+            u_{\mathrm{B},\phi_j}(x_\mathrm{D}, y_\mathrm{D}) \\right \}}
+            {u_0(l_\mathrm{D})}
+            \exp \! \\left[i k_\mathrm{m}(M - 1) \cdot
+            (z_{\phi_j}-l_\mathrm{D}) \\right]
+            \\right \} 
+            \\right \}
+
+    with a modified rotational operator :math:`D_{-\phi_j}^\mathrm{tilt}`
+    and a different filter in Fourier space 
+    :math:`|k_\mathrm{Dx} \cdot \cos \\theta_\mathrm{tilt}|` when compared
+    to :func:`backpropagate_3d`.
+    
+    
     .. versionadded:: 0.1.2
     
     Parameters
     ----------
     uSin : (A, Ny, Nx) ndarray
-        Three-dimensional sinogram of plane wave recordings
-        :math:`u_{\mathrm{B}, \phi_0}(x_\mathrm{D}, y_\mathrm{D},
-        z_\mathrm{D})`
-        normalized by the amplitude of the unscattered wave :math:`a_0`
+        Three-dimensional sinogram of plane recordings
+        :math:`u_{\mathrm{B}, \phi_j}(x_\mathrm{D}, y_\mathrm{D})`
+        divided by the incident plane wave :math:`u_0(l_\mathrm{D})`
         measured at the detector.
     angles : ndarray of shape (A,3) or 1D array of length A
-        If the shape is (A,3), then `angles` consists of vectors
+        If the shape is (A,3), then ``angles`` consists of vectors
         on the unit sphere that correspond to the direction
         of illumination and acquisition (s₀). If the shape is (A,),
-        then `angles` is  a one-dimensional array of angles [rad]
-        that determines the rotational position in a plane.
-        In both cases, `tilted_axis` must be set according to the
+        then ``angles`` is  a one-dimensional array of angles in radians
+        that determines the angular position :math:`\phi_j`.
+        In both cases, ``tilted_axis`` must be set according to the
         tilt of the rotational axis.
     res : float
         Vacuum wavelength of the light :math:`\lambda` in pixels.
@@ -426,21 +450,26 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
         Only compute the output image at these coordinates. This
         keyword is reserved for future versions and is not
         implemented yet.
-    weight_angles : bool, optional
-        If `True` weight each backpropagated projection with a factor
+    weight_angles : bool
+        If ``True``, weights each backpropagated projection with a factor
         proportional to the angular distance between the neighboring
-        projections.
+        projections. 
+        
+        .. math::
+            \Delta \phi_0 \\longmapsto \Delta \phi_j = \\frac{\phi_{j+1} - \phi_{j-1}}{2}
+        
+        This currently only works when `angles` has the shape (A,).
     onlyreal : bool
-        If `True`, only the real part of the reconstructed image
+        If ``True``, only the real part of the reconstructed image
         will be returned. This saves computation time.
     padding : tuple of bool
         Pad the input data to the second next power of 2 before
         Fourier transforming. This reduces artifacts and speeds up
         the process for input image sizes that are not powers of 2.
-        The default is padding in x and y: `padding=(True, True)`.
+        The default is padding in x and y: ``padding=(True, True)``.
         For padding only in x-direction (e.g. for cylindrical
-        symmetries), set `padding` to `(True, False)`. To turn off
-        padding, set it to `(False, False)`.
+        symmetries), set ``padding`` to ``(True, False)``. To turn off
+        padding, set it to ``(False, False)``.
     padfac : float
         Increase padding size of the input data. A value greater
         than one will trigger padding to the second-next power of
@@ -454,21 +483,32 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
         approximation, where an approximat zero in the phase might
         translate to 2πi due to the unwrapping algorithm. In that
         case, this value should be a multiple of 2πi. 
-        If `padval` is `None`, then the edge values are used for
-        padding (see documentation of `numpy.pad`).
+        If `padval`` is ``None``, then the edge values are used for
+        padding (see documentation of :func:`numpy.pad`).
     order : int between 0 and 5
         Order of the interpolation for rotation.
         See :func:`scipy.ndimage.interpolation.affine_transform` for details.
-    dtype : dtype object or argument for np.dtype
+    dtype : dtype object or argument for :func:`numpy.dtype`
         The data type that is used for calculations (float or double).
-        Defaults to np.float.
+        Defaults to ``numpy.float``.
     num_cores : int
         The number of cores to use for parallel operations. This value
         defaults to the number of cores on the system.
-    jmc, jmm : instance of :func:`multiprocessing.Value` or `None`
+    save_memory : bool
+        Saves memory at the cost of longer computation time.
+        
+        .. versionadded:: 0.1.5
+        
+    copy : bool
+        Copy input sinogram ``uSin`` for data processing. If ``copy``
+        is set to ``False``, then ``uSin`` will be overridden.
+        
+        .. versionadded:: 0.1.5
+        
+    jmc, jmm : instance of :func:`multiprocessing.Value` or ``None``
         The progress of this function can be monitored with the 
-        :mod:`jobmanager` package. The current step `jmc.value` is
-        incremented `jmm.value` times. `jmm.value` is set at the 
+        :mod:`jobmanager` package. The current step ``jmc.value`` is
+        incremented ``jmm.value`` times. ``jmm.value`` is set at the 
         beginning.
     verbose : int
         Increment to increase verbosity.
@@ -492,7 +532,7 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
     Notes
     -----
     This implementation can deal with projection angles that are not
-    distributed along a circle  about the rotation axisal. If there are
+    distributed along a circle  about the rotational axis. If there are
     slight deviations from this circle, simply pass the 3D rotational
     positions instead of the 1D angles to the `angles` argument. In
     principle, this should improve the reconstruction. The general
@@ -505,7 +545,19 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
     of the unit sphere (but this kind of rotation is probably difficult
     to implement experimentally).
     
+    Do not use the parameter `lD` in combination with the Rytov
+    approximation - the propagation is not correctly described.
+    Instead, numerically refocus the sinogram prior to converting
+    it to Rytov data (using e.g. :func:`odtbrain.sinogram_as_rytov`)
+    with a numerical focusing algorithm (available in the Python
+    package :py:mod:`nrefocus`).
     """
+    ne.set_num_threads(num_cores)
+
+    if copy:
+        uSin = uSin.copy()
+        angles = angles.copy()
+
     # `tilted_axis` is required for several things:
     # 1. the filter |kDx*v + kDy*u| with (u,v,w)==tilted_axis
     # 2. the alignment of the rotational axis with the y-axis
@@ -534,30 +586,31 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
     tilted_axis_yz = norm_vec(np.dot(rotmat, tilted_axis))
     
     A = angles.shape[0]
+    angles = np.squeeze(angles) # Allow shapes (A,1)
     assert angles.shape in [(A,), (A,3)], "`angles` must have shape (A,) or (A,3)!"
     # jobmanager
     if jmm is not None:
         jmm.value = A + 2
-
     
-    if len(angles.shape) != 2:
+    if len(angles.shape) == 1:
         if weight_angles:
             weights = util.compute_angle_weights_1d(angles).reshape(-1,1,1)
-        else:
-            weights = 1
         # compute the 3D points from tilted axis
         angles = sphere_points_from_angles_and_tilt(angles, tilted_axis_yz)
     else:
         if weight_angles:
             warnings.warn("3D angular weighting not yet supported!")
-        #TODO:
-        # - make sure angles are normalized
-        weights = 1
+            weights = 1
+
+        # normalize and rotate angles
+        for ii in range(angles.shape[0]):
+            #angles[ii] = norm_vec(angles[ii]) #-> not correct
+            # instead rotate like `tilted_axis` onto the y-z plane.
+            angles[ii] = norm_vec(np.dot(rotmat, angles[ii]))
     
     # check for dtype
     dtype = np.dtype(dtype)
-    if not dtype.name in ["float32", "float64"]:
-        raise ValueError("dtype must be float32 or float64.")
+    assert dtype.name in ["float32", "float64"], "dtype must be float32 or float64!"
 
     assert num_cores <= _ncores, "`num_cores` must not exceed number " +\
                                  "of physical cores: {}".format(_ncores)
@@ -595,9 +648,10 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
     # latter sign convention.
     # This is not a big problem. We only need to multiply the imaginary
     # part of the scattered wave by -1.
+    sinogram = uSin
 
-    # save memory
-    sinogram = uSin*weights
+    if weight_angles:
+        sinogram *= weights
 
     # lengths of the input data
     (la, lny, lnx) = sinogram.shape
@@ -646,6 +700,7 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
 
     # save memory
     del sinogram
+    
     if verbose > 0:
         print("......Image size (x,y): {}x{}, padded: {}x{}".format(
             lnx, lny, sino.shape[2], sino.shape[1]))
@@ -713,8 +768,7 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
 
     # Filter M so there are no nans from the root
     M = 1. / km * np.sqrt((km**2 - kx**2 - ky**2) * filter_klp)
-    # The input data is already divided by a0
-    #prefactor  = -1j * km / ( 2 * np.pi * a0 )
+
     prefactor = -1j * km / (2 * np.pi)
     prefactor *= dphi0
     # Also filter the prefactor, so nothing outside the required
@@ -727,8 +781,11 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
     
     u, v, _w = tilted_axis
     filterabs = np.abs(kx*v+ky*u) * filter_klp
-
-    prefactor *= np.exp(-1j * km * M * lD)
+    # new in version 0.1.4:
+    # We multiply by the factor (M-1) instead of just (M)
+    # to take into account that we have a scattered
+    # wave that is normalized by u0.
+    prefactor *= np.exp(-1j * km * (M-1) * lD)
     # Perform filtering of the sinogram,
     # save memory by in-place operations
     #projection = np.fft.fft2(sino, axes=(-1,-2)) * prefactor
@@ -757,7 +814,7 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
     temp_array, myfftw_plan
 
     projection = sino
-    projection[:] *= prefactor
+    projection[:] *= prefactor / (lNx * lNy)
     projection[:] *= filterabs
 
     # save memory
@@ -795,22 +852,19 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
     #              z, y, x
     Mp = M.reshape(lNy, lNx)
 
-
-    # Compute the filter in Fourier space in parallel on-by one
-    # This saves an enormous amount of memory when compared to
-    # simply executing:
     # filter2 = np.exp(1j * zv * km * (Mp - 1))
-    
-    Mpm1 = km * (Mp - 1)
-    args=zip(zv.flatten(), [Mpm1]*zv.shape[0])
-    filter2_pool = mp.Pool(processes=num_cores)
-    filter2 = filter2_pool.map(_filter2_func, args)
-    filter2_pool.terminate()
-    filter2_pool.terminate()
-    del filter2_pool, args, Mpm1
-
-    # occupies some amount of ram
-    #filter2[0].size*len(filter2)*128/8/1024**3
+    f2_exp_fac = 1j* km * (Mp - 1)
+    if save_memory:
+        # compute filter2 later
+        pass
+    else:
+        # compute filter2 now
+        filter2 = ne.evaluate("exp(factor * zv)",
+                              local_dict={"factor": f2_exp_fac,
+                                          "zv":zv})
+        # occupies some amount of ram, but yields faster
+        # computation later
+        #filter2[0].size*len(filter2)*128/8/1024**3
 
     if jmc is not None:
         jmc.value += 1
@@ -819,7 +873,6 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
     #projection = projection.reshape(la, 1, lNy, lNx)
     projection = projection.reshape(la, lNy, lNx)
 
-
     # This frees comparatively few data
     del M
     #del Mp
@@ -870,19 +923,28 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
         # filter2.shape == (ln, lNx, lNy)
         
         for p in range(len(zv)):
-            inarr[:] = filter2[p] * projection[aa]
+            if save_memory:
+                # compute filter2 here;
+                # this is comparatively slower than the other case
+                ne.evaluate("exp(factor * zvp) * projectioni",
+                            local_dict={"zvp":zv[p],
+                                        "projectioni":projection[aa],
+                                        "factor":f2_exp_fac},
+                            out=inarr)
+            else:
+                # use universal functions
+                np.multiply(filter2[p], projection[aa], out=inarr)
             myifftw_plan.execute()
             filtered_proj[p, :, :] = inarr[
                                            padyl:padyl + lny,
                                            padxl:padxl + lnx
-                                          ] / (lNx * lNy)
+                                          ]
         
         # The Cartesian axes in our array are ordered like this: [z,y,x]
         # However, the rotation matrix requires [x,y,z]. Therefore, we
         # need to np.transpose the first and last axis and also invert the
         # y-axis.
-        filtered_proj = filtered_proj.transpose(2,1,0)
-        filtered_proj[:,:,:] = filtered_proj[:,::-1,:] 
+        filtered_proj = filtered_proj.transpose(2,1,0)[:,::-1,:]
 
         # get rotation matrix for this point and also rotate in plane
         _drot, drotinv = rotation_matrix_from_point_planerot(angles[aa],
@@ -899,21 +961,23 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
         c = 0.5 * np.array(filtered_proj.shape) - .5
         offset = c - np.dot(drotinv, c)
         
-        # perform rotation
-        rotxzr = scipy.ndimage.interpolation.affine_transform(
+        ## Perform rotation
+        # We cannot split the inplace-rotation into multiple subrotations
+        # as we did in _Back_3d_tilted.backpropagate_3d, because the rotation
+        # axis is arbitrarily placed in the 3d array. Rotating single
+        # slices does not yield the same result as rotating the entire
+        # array. Instead of using affine_transform, map_coordinates might
+        # be faster for multiple cores. 
+        outarr.real += scipy.ndimage.interpolation.affine_transform(
                                 filtered_proj.real, drotinv,
                                 offset=offset, mode="constant",
                                 cval=0, order=intp_order)
 
-        
-        outarr.real += rotxzr
-
         if not onlyreal:
-            rotxzi = scipy.ndimage.interpolation.affine_transform(
+            outarr.imag += scipy.ndimage.interpolation.affine_transform(
                                     filtered_proj.imag, drotinv,
                                     offset=offset, mode="constant",
                                     cval=0, order=intp_order)
-            outarr.imag += rotxzi
 
         if jmc is not None:
             jmc.value += 1
@@ -927,7 +991,6 @@ def backpropagate_3d_tilted(uSin, angles, res, nm, lD,
     gc.collect()
 
     # Undo the axis transposition that we performed previously.
-    outarr = outarr.transpose(2,1,0)
-    outarr[:,:,:] = outarr[:,::-1,:]
+    outarr = outarr.transpose(2,1,0)[:,::-1,:]
 
     return outarr
